#!/usr/local/bin/perl -w
########################################

# 日本語名：更新検知ツール
# 機能：	Webサイトの更新検知


# モジュール
use strict;
use warnings;
use Log::Log4perl;
use lib qw(/var/detect/detect_update/pm);
use XML::TreePP;
use Tie::IxHash;
use Encode;
use utf8;
use encoding "utf8";
use open IN  => ":utf8";
use open OUT => ":utf8";
use open IO => ":utf8";
use Jcode;
use LWP::UserAgent;
use HTTP::Request::Common( "GET" );

# 自作モジュール
use DetectUpdate;
use Item;
use SubItem;
use MailTransfer;

&main();

exit;

########################################

sub main {

	# ログ出力の準備
	Log::Log4perl->init(DetectUpdate::LOG_CONF);
	my $log = Log::Log4perl->get_logger();

	$log->debug("START --------------------------------------------------");

	# アイテムの更新を確認

	# アイテムテーブルからレコード取得
	my $item_iterator = Item->retrieve_all();

	if (!defined($item_iterator)) {

		$log->debug("[Error1] Cannot Get Item Record");
		return;

	}


	# 更新履歴を格納(各要素には、各サブアイテムの更新履歴がハッシュのリファレンスとして格納)
	my @Update_l = ();
	my @Results_l = ();
	my $xml = ();

	my %Item_update_h;	# 更新情報を加工する
	my %Update_Report_h;
	my %Item_Info_h;

	my @Updated_items_l;

	tie %Item_update_h, 'Tie::IxHash';		# ハッシュが順番を持つようにする
	tie %Update_Report_h, 'Tie::IxHash';	# ハッシュが順番を持つようにする
	tie %Item_Info_h, 'Tie::IxHash';

	while (my $item_record = $item_iterator->next) {	# アイテムレコードの件数繰り返す

		# アイテムID、URL取得
		my $item_id = $item_record->item_id;
		my $item_name = $item_record->item_name;
		my $item_url = $item_record->item_URL;
		my $follow_link = $item_record->follow_link;
		my $user_id = $item_record->user_id;
		my $password = $item_record->password;
		my $parameter = $item_record->parameter;
		my $comment = $item_record->comment;

		####################################################################################################
		# 100918 修正
		# アイテムのTOPページのリンクが生きているか確認し、死んでいればメール送信
		####################################################################################################

		my $user_agent = LWP::UserAgent->new();
		$user_agent->timeout(10);

		# id & passwd
		my $request = GET($item_url);
		$request->authorization_basic( $user_id, $password ) if (defined($user_id) && defined($password));

		# proxy
		#my $proxy = DetectUpdate::PROXY;
		#$user_agent->proxy("http", $proxy);

		my $response = $user_agent->request($request);

		# status code がエラーであればメール送信
		if ($response->is_error) {

			my $err_code = $response->code;
			my $gui_url = DetectUpdate::GUI_URL;

			my $message = "MEDALS 管理者各位\n\n下記サイトのHTTPステータスコードがエラーです。\n\n";
			$message .= "ステータスコード : $err_code\nアイテム番号 : $item_id\nアイテム名 : $item_name\nURL : $item_url\n\n";
			$message .= "＊このメールは、MEDALS 運用ツールから自動送信しています。\n\n管理GUI: $gui_url";

			# 件名、宛先など
			my $from_mail_addr = MailTransfer::FROM;
			my $to_mail_addr  = MailTransfer::TO;
			my $cc_mail_addr  = MailTransfer::CC;
			my $bcc_mail_addr = MailTransfer::BCC;
			my $subject = MailTransfer::SUB_DEAD_LINK_WARN . " $item_id";
			#my $subject = "hoge";

			# メール送信
			&MailTransfer::send_mail($subject, $message, $from_mail_addr, $to_mail_addr, $cc_mail_addr, $bcc_mail_addr);

		}
		####################################################################################################

		my %Item_Infomation_h = (

						'ItemId' => $item_id, 
						'URL' => $item_url, 
						'Link' => $follow_link, 
						'UserId' => $user_id, 
						'PassWord' => $password, 
						'Parameter' => $parameter

					);


		# アイテムの更新を検知
		my $item_update = DetectUpdate::subItem(\%Item_Infomation_h);



		if ( defined($item_update) ) {	# 更新が検知されたら真

			# 現在の日付を取得

			my $time = shift @$item_update;
			my %time = %$time;
			my $date = "$time{'Year'}/$time{'Month'}/$time{'Day'}($time{'Wday'}) $time{'Hour'}:$time{'Minute'}:$time{'Second'}";


			# XML出力
			my $treepp = XML::TreePP->new(use_ixhash => 1);

			# XMLファイル名
			my $xml_file = $item_id.'.xml';

			# ファイルが存在する場合、Update要素を追加
			if ( -e DetectUpdate::UPDATE_REPORTS_DIR.'/'.$xml_file ) {

				my $tree = $treepp->parsefile( DetectUpdate::UPDATE_REPORTS_DIR.'/'.$xml_file );

				my $item_id_element = $tree->{'Update_Report'}->{'ItemInfo'}->{'Item_Id'};
				my $item_url_element = $tree->{'Update_Report'}->{'ItemInfo'}->{'Item_URL'};
				my $item_name_element = $tree->{'Update_Report'}->{'ItemInfo'}->{'Item_Name'};

				my $results = $tree->{'Update_Report'}->{'Results'};

				my $reffrence_type = ref($results);

				if ($reffrence_type eq 'ARRAY') {

					@Results_l = @$results;

					push ( @Results_l, { -Date => $date, Content => $item_update } );

				} elsif ($reffrence_type eq 'HASH') {

					push ( @Results_l, $results->{'Update'} );
					push ( @Results_l, { -Date => $date, Content => $item_update } );

				}

				%Item_Info_h = ( Item_Id => $item_id_element, Item_Name => $item_name_element, Item_URL => $item_url_element );

				%Item_update_h  = ( ItemInfo=>\%Item_Info_h, Results=>{ Update=>[@Results_l] } );

				%Update_Report_h = ( Update_Report => \%Item_update_h );

				$xml = $treepp->write( \%Update_Report_h );


			# XMLファイルが存在しない場合
			} else {

				push ( @Results_l, { -Date => $date, Content => $item_update } );

				my $data = $item_record->item_name;

				%Item_Info_h = ( Item_Id => $item_record->item_id, Item_Name => $item_record->item_name, Item_URL => $item_url );

				%Item_update_h = ( ItemInfo=>\%Item_Info_h, Results=>{ Update=>[@Results_l] } );

				%Update_Report_h = ( Update_Report => \%Item_update_h );
				$xml = $treepp->write( \%Update_Report_h );

			}

			# ファイルへ書き出す
			open (XML, '+>'.DetectUpdate::UPDATE_REPORTS_DIR.'/'.$xml_file) || $log->debug("[Error11] XML Open Error -> $xml_file");
			print XML $xml;
			close (XML) || $log->debug("[Error12] XML Close Error -> $xml_file");

			my $chmod_target = DetectUpdate::UPDATE_REPORTS_DIR.'/'.$xml_file;
			open (CHMOD, "chmod 777 $chmod_target |");
			close(CHMOD);


			# 更新されたアイテムを配列に格納

			chomp($item_id, $item_name, $item_url, $date);


			##### コメントの整形  #####

			my @Pre_comment_rows_l = split (/\n/, $comment);	# コメントを改行で分ける

			# 各行を50バイトずつ分ける
			my @Comment_rows_l = ();

			foreach my $pre_row (@Pre_comment_rows_l) {

				my $jcode = new Jcode($pre_row);

				foreach ( $jcode->jfold(50) ){

					push(@Comment_rows_l, $_);

				}

			}

			####################################


			my %Updated_items_h = (

				Id => $item_id, 
				Name => $item_name, 
				Url => $item_url, 
				Time => $date,
				Comment => \@Comment_rows_l

			);

			push (@Updated_items_l, \%Updated_items_h);

			# 初期化
			%Item_update_h = ();
			%Update_Report_h = ();
			@Results_l = ();
			$xml = ();
			%Item_Info_h = ();

		}

	}


	# メール送信

	if (@Updated_items_l) {

		my $message = "MEDALS 管理者各位\n\n以下のサイトの更新が検知されました。\n\n";

		# コメント用変数追加	091214
		my ($row_id, $row_name, $row_url, $row_time4, $comment, $row_comment, $first_row) = '';
		my @Row_comment_l = ();

		my $number = 1;

		foreach (@Updated_items_l)  {

			$row_id = "　【$number】アイテム番号　: $_->{'Id'}\n";
			$row_name = "　　　 アイテム名　　: $_->{'Name'}\n";
			$row_url = "　　　 URL　　　　　 : $_->{'Url'}\n";
			$row_time4 = "　　　 検知時刻　　　: $_->{'Time'}\n";

			########## コメントを追加	091210 ##########

			$comment = $_->{'Comment'};
			@Row_comment_l = @$comment;



			$first_row = shift (@Row_comment_l);

			unless (defined($first_row)) {

				$first_row = '';

			}


			$row_comment = "　　　 コメント　　　: $first_row\n";

			foreach (@Row_comment_l) {

				$row_comment = $row_comment."　　　　　　　　　　   $_\n";

			}

			$row_comment = $row_comment."\n";

			$message = $message.$row_id.$row_name.$row_url.$row_time4.$row_comment;

			#################################################

			$number++;

		}

		my $gui_url = DetectUpdate::GUI_URL;
		$message = $message."＊このメールは、MEDALS 運用ツールから自動送信しています。\n\n管理GUI: $gui_url";


		# 件名、宛先など
		my $from_mail_addr = MailTransfer::FROM;
		my $to_mail_addr  = MailTransfer::TO;
		my $cc_mail_addr  = MailTransfer::CC;
		my $bcc_mail_addr = MailTransfer::BCC;
		my $subject = MailTransfer::SUBJECT;

		# メール送信
		&MailTransfer::send_mail($subject, $message, $from_mail_addr, $to_mail_addr, $cc_mail_addr, $bcc_mail_addr);

	} 

	$log->debug("END --------------------------------------------------");

}	#	main


