####################################################################################################
# ファイル名：DetectUpdate.pm
# 機能：
# 更新検知ツールで使用するモジュール
# 
# 改訂履歴：
# 1.0.1：新規作成	2009/12/14
# 
####################################################################################################
package DetectUpdate;

use strict;
use warnings;
use utf8;
use Digest::MD5 qw(md5 md5_hex md5_base64);				# MD5計算
use Item;								# アイテムテーブル操作
use SubItem;								# サブアイテムテーブル操作
use UpdateHistry;
use Log::Log4perl;							# ログ出力
use Tie::IxHash;							# ハッシュの順序付け
use Encode;								# エンコード
use LWP::UserAgent;							# HTTPアクセス
use File::Find;								# ファイル操作
use File::Path;								# ファイル操作
use URI;
use HTTP::Request::Common( "GET" );
use HTML::TagParser;
use MakeDiff;
use File::Basename;

use constant ACCESS_DELAY=>0;								# サイトにアクセスする間隔(秒)
use constant WGET_OUTPUT_DIR=>'/var/detect/gpmanager/mirror_site';			# ミラーサイトを作るディレクトリ
use constant UPDATE_REPORTS_DIR=>'/var/detect/detect_update/update_reports';			# 更新情報のXMLファイルを出力するディレクトリ
use constant PROXY=>'http://20366471:moleskin240@202.250.74.8:8080';								# proxy
use constant WGET_LOG_FILE=>'/var/detect/detect_update/log/wget.txt';				# wgetのログファイル
use constant LOG_CONF=>'/var/detect/detect_update/log/conf/log4perl.conf';			# ログ出力の設定ファイル
use constant GUI_URL=>'https://*****/gpmanager/index.cgi';								# 管理GUIのURL
#use constant WGET_CMD=>'/usr/bin/wget'; # wgetコマンド
my $WGET_CMD='/usr/local/bin/wget'; # wgetコマンド

####################################################################################################
# 関数名：get_time
# 機能 ：現在時刻を格納したハッシュを返す
# 戻り値：
####################################################################################################

sub get_time {



	my @time = localtime(time);

	my $year = $time[5] + 1900;
	my $month = $time[4] + 1;
	my $day = $time[3];
	my %wday = ( 
					0=>'Sun', 
					1=>'Mon', 
					2=>'Tue', 
					3=>'Wed', 
					4=>'Thu', 
					5=>'Fri', 
					6=>'Sat'
				);

	my $wday = $wday{$time[6]};

	my $hour = $time[2];
	my $minute = $time[1];
	my $second = $time[0];

	my %time = (

					'Year' => $year,
					'Month' => $month, 
					'Day' => $day, 
					'Wday' => $wday, 
					'Hour' => $hour, 
					'Minute' => $minute, 
					'Second' => $second

				);

	return \%time;

}



####################################################################################################
# 関数名：get_content_infomation
# 機能 ：引数のURLにアクセスし、HTTPヘッダ情報を取得する
# 戻り値：
####################################################################################################

sub get_content_infomation {

	Log::Log4perl->init(LOG_CONF);
	my $log = Log::Log4perl->get_logger();

	# コンテンツ情報を取得したいURLを受け取る
	my $url = shift @_;
	my $user_id = shift @_;			# ユーザID
	my $password = shift @_;		# パスワード

#	print "\t\t\t$url\n";	# debug

	# LWPユーザエージェントをnewする
	my $user_agent = LWP::UserAgent->new();
	$user_agent->timeout(10);

	my $request = GET($url);
	$request->authorization_basic( $user_id, $password ) if (defined($user_id) && defined($password));

	# プロキシ設定
	my $proxy = PROXY;
	$user_agent->proxy("http", $proxy);


	# GETでURLにアクセス
#	my $response = $user_agent->get($url);
	my $response = $user_agent->request($request);

	sleep(ACCESS_DELAY);


	# ハッシュにヘッダ情報を設定

	if ($response->is_success) {

		my $code = $response->code;
		my $content_type = $response->header("Content-Type");
		my $location = $response->header("Location");
		my $last_modified = $response->header("Last-Modified");
		my $content_length = $response->header("Content-Length");
		my $etag = $response->header("ETag");


		################################### MD5計算	091214 ###################################

		my $content = $response->content;

		my $md5 = ();

		if ($content_type =~ /text\/html/) {

			my $html = HTML::TagParser->new();

			eval{

				$html->parse($content);
				my $elem = $html->getElementsByTagName( "body" );

				if (defined($elem)) {

					my $text = $elem->innerText();
					$md5 = md5_hex( Encode::encode_utf8($text) );

				} else {

					$md5 = md5_hex( Encode::encode_utf8($content) );

				}

			};

			if ($@) {

				$md5 = md5_hex( Encode::encode_utf8($content) );
				$log->debug("[Warning] $@ -> $url");

			}

		} else {

			$md5 = md5_hex($content);

		}



		########################################################################################


		my %Content_infomation_h = (

			'Code' => $code, 
			'Content-Type' => $content_type, 
			'Location' => $location,
			'Last-Modified' => $last_modified, 
			'ETag' => $etag,
			'MD5' => $md5

		);

		# ヘッダ情報のハッシュを返す
		return (\%Content_infomation_h);

	} else {

		# アクセス失敗

		my $error_code = $response->code;

		$log->debug("[Error3] HTTP Request Error at Getting Content Infomation -> Code: $error_code, URL: '$url'");

		return undef;

	}


}


####################################################################################################
# 関数名：mirror
# 機能 ：ミラーリング、及びサブアイテムテーブルへの登録
# 戻り値：1アイテムレベルの更新情報
####################################################################################################

sub mirror {

	#ログ出力の準備
	Log::Log4perl->init(LOG_CONF);
	my $log = Log::Log4perl->get_logger();

	my $argment = shift @_;
	my %Item_Infomation_h = %$argment;

	my $item_id = $Item_Infomation_h{'ItemId'};			# アイテム番号
	my $item_url = $Item_Infomation_h{'URL'};			# ミラーリングの対象URL
	my $follow_link = $Item_Infomation_h{'Link'};			# リンク辿る階層
	my $user_id = $Item_Infomation_h{'UserId'};			# ユーザID
	my $password = $Item_Infomation_h{'PassWord'};			# パスワード
	my $parameter = $Item_Infomation_h{'Parameter'};		# パラメータ


	# URLのチェック 091214 ###########################################################################################

	if ( !($item_url =~ /^\s*http\:\/\//) && !($item_url =~ /^\s*ftp\:\/\//) && !($item_url =~ /^\s*https\:\/\//) ) {

		$log->debug("[Error20] URL format error -> Id: $item_id, URL: '$item_url'");

		$item_url = 'http://';

	}

	##################################################################################################################



	# リダイレクト対策

	my $user_agent = LWP::UserAgent->new();

	$user_agent->timeout(10);

	my $request = GET($item_url);
	$request->authorization_basic( $user_id, $password ) if (defined($user_id) && defined($password));


	# プロキシ設定
	my $proxy = PROXY;
	$user_agent->proxy("http", $proxy);

#	my $response = $user_agent->get($item_url);
	my $response = $user_agent->request($request);

	sleep(ACCESS_DELAY);

	my $max = 0;	# リダイレクト調査で辿る、リンクの限度

	if ($response->is_success) {

		while ( ( defined($response->header("Refresh")) || defined($response->previous) ) && $max < 10) {

			if ( defined($response->header("Refresh")) ) {

				my $refresh_header = $response->header("Refresh");

				if ( $refresh_header =~ m|url\=(.+)$|i ) {

					my $str = $1;
					my $base_uri = $item_url;

					my $uri = URI->new_abs( $str, $base_uri );

					$item_url = $uri->as_string;

					my $item_record = Item->retrieve($item_id);
					$item_record->item_URL($item_url);
					$item_record->update;

				} else {

					last;

				}

				$response = $user_agent->get($item_url);

			}

			if ( defined($response->previous) ) {

				my $previous_code = $response->previous->code;
				my $previous_location = $response->previous->header("Location");

				if ( $previous_code =~ /301/ ) {

					############################ 相対パス対策	091211 ##########################

					my $str = $previous_location;
					my $base_uri = $item_url;

					my $uri = URI->new_abs( $str, $base_uri );

					$item_url = $uri->as_string;

					#############################################################################

					my $item_record = Item->retrieve($item_id);
					$item_record->item_URL($item_url);
					$item_record->update;

				}

			}

			$response = undef;
			$response = $user_agent->get($item_url);
			sleep(ACCESS_DELAY);

			unless ($response->is_success) {

				last;

			}

		$max++;

		}

	} else {

		my $error_code = $response->code;
		$log->debug("[Error4] HTTP Request Error at Checking Redirect -> Code: $error_code, Id: $item_id, URL: '$item_url'");

	}



	# 現在の時刻を取得
	my $return_get_time = &get_time;
	my %time = %$return_get_time;

	# 時刻の整形(9時 => 09時 の形式にする)	20091214 #############################################
	if ($time{'Month'} < 10) {

		$time{'Month'} = "0".$time{'Month'};

	}

	if ($time{'Day'} < 10) {

		$time{'Day'} = "0".$time{'Day'};

	}

	if ($time{'Hour'} < 10) {

		$time{'Hour'} = "0".$time{'Hour'};

	}

	if ($time{'Minute'} < 10) {

		$time{'Minute'} = "0".$time{'Minute'};

	}

	if ($time{'Second'} < 10) {

		$time{'Second'} = "0".$time{'Second'};

	}

	################################################################################################

	my $date = "$time{'Year'}-$time{'Month'}-$time{'Day'}_$time{'Hour'}-$time{'Minute'}-$time{'Second'}";

	# ログやミラーサイトを出力する、ディレクトリやファイル名
	my $wget_log_file = WGET_LOG_FILE;

	my $mirror_dir = WGET_OUTPUT_DIR;
	my $mirror_site = "$mirror_dir/$item_id/$date";


	# パラメータ設定
	my $post = '';

	if (defined($parameter)) {

		$post = '--post-data='."'$parameter'";

	}


	# 認証設定

	my $basic_id = '';

	if ($user_id) {

		$basic_id = '--http-user='."'$user_id'";

	}


	my $basic_psw = '';


	if ($password) {

		$basic_psw = '--http-password='."'$password'";

	}

	# 辿る階層

	my $link = '';

	if ($follow_link > 0) {

		$link = '-r -l '.$follow_link;

	}

	################### ミラーサイトを新規作成の場合、アクセス権限を777に設定(管理GUIから操作できるように)  091214 #################


	unless (-d "$mirror_dir/$item_id") {
		
		mkdir ("$mirror_dir/$item_id", 0777);
		open (CHMOD, "chmod -R 777 $mirror_dir/$item_id |") || $log->debug("[Error] chmod command Error -> Id: $item_id");
		close(CHMOD);

	}

	################################################################################################################################


	# ミラーリング
	unlink $wget_log_file if (-e $wget_log_file);
	open (WGET, "$WGET_CMD --timeout=10 -t 1 --output-file=$wget_log_file -nv $post $basic_id $basic_psw $link -k -p -E -P $mirror_site $item_url |") 
    || $log->debug("[Error5] Wget Command Error -> Id: $item_id, URL: $item_url");

	close (WGET) || $log->debug("[Error5] Wget Command Error -> Id: $item_id, URL: $item_url");


	# wgetログファイルの読み込み

	open ( WGET_LOG, '<'.$wget_log_file ) || $log->debug("[Error6] Wget Log Open Error -> Id: $item_id, URL: $item_url");

	my @Wget_log_l = <WGET_LOG>;

	close (WGET_LOG) || $log->debug("[Error7] Wget Log Close Error -> Id: $item_id, URL: $item_url");

	# ログファイルから、アクセスしたファイルのURLを取得
	my @Access_url_l       = ();
	my $download_item_file = "";

	foreach (my $i = 0; $i < @Wget_log_l; $i++) {

		if ( $Wget_log_l[$i] =~ /(\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2} URL:)(.+?)( \[.*\]) -> "(.+?)"/ ) {

			push (@Access_url_l, $2);
			$download_item_file = $4 if ($i == 0);	

		} elsif ( $Wget_log_l[$i] =~ /(\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2} )(ERROR.*)/ && $i > 0) {

			my $j = $i;

			$j = $j - 1;

			chomp($Wget_log_l[$j]);

			my $error_code = $2;

			chomp($error_code);

			if ($Wget_log_l[$j] !~ /robots.txt/) {

				$log->debug("[Error8] Mirroring Error of 'Id: $item_id ''URL: $item_url' -> $2 $Wget_log_l[$j]");

			}

		}

	}


	# 1ファイルを1サブアイテムとし、サブアイテムテーブルへ登録

	foreach (@Access_url_l) {
		$log->debug(" loop $_ ");

		# ヘッダ情報をハッシュで取得
		my $return = &get_content_infomation($_, $user_id, $password);

		if ($return) {

			my %Content_infomation_h = %$return;

			my $last_modified  = $Content_infomation_h{'Last-Modified'};
			my $content_length = $Content_infomation_h{'Content-Length'};
			my $e_tag          = $Content_infomation_h{'ETag'};
			my $md5            = $Content_infomation_h{'MD5'};


			# サブアイテムDBへの登録
			SubItem->insert({

							sub_item_URL=>$_, 
							last_modified=>$last_modified, 
							content_length=>$content_length, 
							e_tag=>$e_tag, 
							MD5=>$md5, 
							item_id=>$item_id 

							}) 

            || $log->debug("[Error9] Cannnot Insert Into SubItem -> $_");

		} else {

			# MD5ハッシュ値を取得できなかった

			$log->debug("[Error10] Cannot Get MD5 at Mirroring -> URL: $_");

		}

	}
	
	warn "dame" unless (@Access_url_l);
	if (@Access_url_l) {
		my $update_date = "$time{'Year'}$time{'Month'}$time{'Day'}";
		my $update_time = "$time{'Hour'}$time{'Minute'}$time{'Second'}";
		UpdateHistry->insert({
						item_id=>$item_id,
						date_update=>$update_date,
						time_update=>$update_time
						}) 
		|| $log->debug("[Error9] Cannnot Insert Into UpdateHistry  -> $item_id, $update_date, $update_time");
	}

	

	# ミラーサイトへのリンク（管理ＧＵＩ側で使用）を、link.txtに書き込んでおく
	my ( $first_mirrored_file, $second_mirrored_file ) = &make_url($item_url);	# アイテムのＵＲＬの末尾から、ミラーリング後のファイル名を取得

	my $link_file_full_path = "$mirror_site"."/"."link.txt";

	if ( open (LINK, ">>$link_file_full_path") ) {

		if ( ! defined($second_mirrored_file) ) {

			print LINK "1\t$first_mirrored_file\n";

		} else {

			print LINK "2\t$second_mirrored_file\n";

		}

		print LINK "$download_item_file\n";


		unless ( close(LINK) ) {

			$log->debug("[Error101] cannot close $link_file_full_path");

		}

		&make_diff($mirror_dir, $item_id, $log);

	} else {

		$log->debug("[Error100] cannot open $link_file_full_path");

	}

      

	return \%time;

}

sub make_diff() {
	my ($mirror_dir, $item_id, $log) = @_;

	$mirror_dir = "$mirror_dir/$item_id";

	my $dh;
	unless(opendir($dh, $mirror_dir)) {
		$log->debug("[Error200] cannot opendir $mirror_dir");
		return;
	}
	my @dir_list = grep {/\d{4}-\d{2}-\d{2}_\d{2}-\d{2}-\d{2}/} readdir ($dh);
	my ($new_dir, $old_dir) = sort {$b cmp $a} @dir_list;
	closedir($dh);
	
	my $new_link_file = "$mirror_dir/$new_dir/link.txt";
	my $old_link_file = "$mirror_dir/$old_dir/link.txt";

	my $new_item_file = &get_item_file_name($new_link_file, $log);
	return unless ($new_item_file);

	my $pre_old_dir = "$mirror_dir/$old_dir";
	my $old_item_file = &get_item_file_name($old_link_file, $log, $pre_old_dir);
	return unless ($old_item_file);

	if ($old_item_file =~ m/\/$/) {
		my ($name, $path) = fileparse($new_item_file);
		$old_item_file .= $name;
	}

	my $rc;
	my $message;

	if ($old_item_file =~ m/\.[hH][tT][mM][lL]?$/ and 
			$new_item_file =~ m/\.[hH][tT][mM][lL]?$/) {
		($rc, $message) = &MakeDiff::make_diff_html($old_item_file,
		                                            $new_item_file);
		$log->debug("$message") if ($rc);
	}

	($rc, $message) = &MakeDiff::make_diff_txt($old_item_file, $new_item_file);
	$log->debug("$message") unless ($rc);
	
	($rc, $message) = &MakeDiff::make_diff_rss($old_item_file, $new_item_file);
	$log->debug("$message") if ($rc);
}

sub get_item_file_name() {
	my ($link_file, $log, $pre_dir) = @_;

	my $fh;
	unless ((open $fh, $link_file)) {
		$log->debug("Error[201] cannot open $link_file");
		return "";
	}
	my ($first_line, $second_line) = <$fh>;
	close ($fh);

	my $code;
	my $file_name;

	if ($second_line) {
		chomp $second_line;
		$file_name = $second_line;
	} else {
		# 古いLinkファイルの場合、downloadしたファイル名を記録していないので
		# URLからファイル名を推測する
		chomp $first_line;
		($code, $file_name) = split /\t/, $first_line;
		$file_name =~ s/#.*$//g;
		if ($code == 2) {
			$file_name = "$file_name.html" unless(-e $file_name);
		}
		$file_name = "$pre_dir/$file_name";
	}

	return $file_name;
}

####################################################################################################
# 関数名：subItem
# 機能 ：引数のURLのページの更新を確認する
# 戻り値：
####################################################################################################

sub subItem {

	Log::Log4perl->init(LOG_CONF);
	my $log = Log::Log4perl->get_logger();

	my $argment = shift @_;
	my %Item_Infomation_h = %$argment;

	my $item_id = $Item_Infomation_h{'ItemId'};		# アイテム番号
	my $item_url = $Item_Infomation_h{'URL'};		# アイテムURL
	my $user_id = $Item_Infomation_h{'UserId'};			# ユーザID
	my $password = $Item_Infomation_h{'PassWord'};			# パスワード

	# アイテムレベルで更新を格納
	my @Update_history_l = ();

	#サブアイテムテーブルから、サブアイテムを取得
	my $sub_item_iterator = SubItem->search(item_id => $item_id);


	if ($sub_item_iterator) {	# サブアイテムレコードが存在すれば真

		# サブアイテムレコードの件数繰り返す
		while (my $sub_item_record = $sub_item_iterator->next) {

			my $sub_item_url = $sub_item_record->sub_item_URL;	# 検知対象のサブアイテムURL
			my $sub_item_id = $sub_item_record->sub_item_id;


			# サブアイテムURLにアクセスし、MD5ハッシュ値、ヘッダ情報を取得
			my $return = &get_content_infomation($sub_item_url, $user_id, $password);

			if (defined($return)) {

				my %Content_infomation_h = %$return;


				# 更新されたかを示すフラグ
				my $update_flag = 0;

				# 各情報の更新情報
				my $last_modified_update = undef;
				my $content_length_update = undef;
				my $e_tag_update = undef;
				my $md5_update = undef;


				# サブアイテムレコードから、前回のヘッダ情報を取得
				my $last_modified = $sub_item_record->last_modified;
				my $content_length = $sub_item_record->content_length;
				my $e_tag = $sub_item_record->e_tag;
				my $md5 = $sub_item_record->MD5;


				#比較処理

				my $new_md5_debug = $Content_infomation_h{'MD5'};	# debug
#				print "\t$sub_item_url: $md5   $new_md5_debug\n";					# debug

				if ($Content_infomation_h{'MD5'}) {

					if ( $Content_infomation_h{'MD5'} ne $md5 ) {

#						print "\t\tdifferent\n";	# debug

						$update_flag = 1;
						$md5_update = {new=>$Content_infomation_h{'MD5'}, old=>$md5};


						# ヘッダ情報も、情報として持っておく

						if (defined($Content_infomation_h{'Last-Modified'})) {

							$last_modified_update = {new=>$Content_infomation_h{'Last-Modified'}, old=>$last_modified };

						}

						if (defined($Content_infomation_h{'Content-Length'})) {

							$content_length_update = {new=>$Content_infomation_h{'Content-Length'}, old=>$content_length };

						}

						if (defined($Content_infomation_h{'ETag'})) {

							$e_tag_update = {new=>$Content_infomation_h{'ETag'}, old=>$e_tag };

						}


					}

				}

				my %update;


				# サブアイテムが更新されていれば
				if ($update_flag == 1) {

					tie %update, 'Tie::IxHash';	# ハッシュを順番に格納

					%update = (

						'Content-Type' => $Content_infomation_h{'Content-Type'}, 
						'URL' => $sub_item_url, 
						'MD5' => $md5_update

					);


					if (defined($last_modified_update)) {

						$update{'Last-Modified'} = $last_modified_update;

					}

					if (defined($content_length_update)) {

						$update{'Content-Length'} = $content_length_update;

					}

					if (defined($e_tag_update)) {

						$update{'ETag'} = $e_tag_update;

					}


					# @Update_history_l に、更新履歴を格納する

					push (@Update_history_l, \%update);

				}

			} else {

				# MD5取得できなかった

				$log->debug("[Error2] Cannot Get MD5 at Detecting Update -> URL: '$sub_item_url' ");

			}

		}	# サブアイテムレコードの件数繰り返す



		# サブアイテムが、1つでも更新された場合
		if (@Update_history_l) {

			my $update_num = @Update_history_l;	
			# 同じアイテムに属するサブアイテムレコードを、全て削除

			SubItem->search(item_id => $item_id)->delete_all;

			# ミラーリング
			my $return_mirror = &mirror(\%Item_Infomation_h);

			my %time = %$return_mirror;

			unshift(@Update_history_l, \%time);

			# アイテムレベルの更新情報を返す
			return \@Update_history_l;


		# サブアイテムが1つも更新されなかった場合
		} else {

			# nullを返す
			return undef;

		}



	} else {	# サブアイテムレコード1件もないなら(アイテムに初めてアクセスなら)

		# ミラーリングし、nullを返す

		&mirror(\%Item_Infomation_h);

		return undef;

	}

}




########################################################################################
# 関数名:make_url
# 機能  :ＵＲＬの作成
# 引数  :URL
# 戻り値:編集後URL
########################################################################################

sub make_url {
	my($str);
	my($path);
	my($second_path);
	
	# 引数受取
	$str = shift(@_);
	my @list = split(/\/\//, $str, 2);
	my $s_str = $list[1];
	chomp($s_str);
	
	if( $s_str =~ /(.+)\.html$/ || $s_str =~ /(.+)\.html\#(.+)/ ) {	# 拡張子が.htmlまたは.html#ラベル
		$path = $s_str;

	} elsif( $s_str =~ /(.+)\.html\?(.+)/ ) { # 拡張子が.html?パラメータまたは.html?パラメータ#ラベル
		$path = $1.'.html%3f'.$2.'.html';

	} elsif( $s_str =~ /(.+)\.html\?(.+)\#(.+)/ || $s_str =~ /(.+)\.html\?(.+)/ ) { # 拡張子が.html?パラメータまたは.html?パラメータ#ラベル
		$path = $1.'.html%3f'.$2;
	} elsif( $s_str =~ /(.+)\/$/ ) {								# /で終わるもの
		$path = $s_str;
	} elsif( $s_str =~ /(.+)\.htm\#(.+)/ ) {						# 拡張子が.htm#ラベル
		$path = $1.'.htm.html#'.$2;
	} elsif( $s_str =~ /(.+)\.htm$/ ) {								# 拡張子が.htm
		$path = $1.'.htm.html';
	} elsif($s_str =~ /(.+)\.htm\?(.+)\#(.+)/ ) {					# 拡張子が.htm?パラメータ#ラベル
		$path = $1.'.htm%3f'.$2.'.html#'.$3;
	} elsif($s_str =~ /(.+)\.htm\?(.+)/) {							# 拡張子が.htm?パラメータ
		$path = $1.'.htm%3f'.$2.'.html';
	} elsif( $s_str =~ /(.+)\.jsp\#(.+)/ ) {						# 拡張子が.jsp#ラベル
		$path = $1.'.jsp.html#'.$2;
	} elsif($s_str =~ /(.+)\.jsp$/) {								# 拡張子が.jsp
		$path = $1.'.jsp.html';
	} elsif($s_str =~ /(.+)\.jsp\?(.+)\#(.+)/ ) {					# 拡張子が.jsp?パラメータ#ラベル
		$path = $1.'.jsp%3f'.$2.'.html#'.$3;
	} elsif($s_str =~ /(.+)\.jsp\?(.+)/) {							# 拡張子が.jsp?パラメータ
		$path = $1.'.jsp%3f'.$2.'.html';
	} elsif( $s_str =~ /(.+)\.php\#(.+)/ ) {						# 拡張子が.php#ラベル
		$path = $1.'.php.html#'.$2;
	} elsif($s_str =~ /(.+)\.php$/) {								# 拡張子が.php
		$path = $1.'.php.html';
	} elsif($s_str =~ /(.+)\.php\?(.+)\#(.+)/ ) {					# 拡張子が.php?パラメータ#ラベル
		$path = $1.'.php%3f'.$2.'.html#'.$3;
	} elsif($s_str =~ /(.+)\.php\?(.+)/) {							# 拡張子が.php?パラメータ
		$path = $1.'.php%3f'.$2.'.html';
	} elsif($s_str =~ /(.+)\.cgi\#(.+)/ ) {							# 拡張子が.cgi#ラベル
		$path = $1.'.cgi.html#'.$2;
	} elsif($s_str =~ /(.+)\.cgi$/) {								# 拡張子が.cgi
		$path = $1.'.cgi.html';
	} elsif($s_str =~ /(.+)\.cgi\?(.+)\#(.+)/ ) {					# 拡張子が.cgi?パラメータ#ラベル
		$path = $1.'.cgi%3f'.$2.'.html#'.$3;
	} elsif($s_str =~ /(.+)\.cgi\?(.+)/) {							# 拡張子が.cgi?パラメータ
		$path = $1.'.cgi%3f'.$2.'.html';
	} elsif($s_str =~ /(.+)\.do\#(.+)/ ) {							# 拡張子が.do#ラベル
		$path = $1.'.do.html#'.$2;
	} elsif($s_str =~ /(.+)\.do$/) {								# 拡張子が.do
		$path = $1.'.do.html';
	} elsif($s_str =~ /(.+)\.do\?(.+)\#(.+)/ ) {					# 拡張子が.do?パラメータ#ラベル
		$path = $1.'.do%3f'.$2.'.html#'.$3;
	}elsif($s_str =~ /(.+)\.do\?(.+)/) {							# 拡張子が.do?パラメータ
		$path = $1.'.cgi%3f'.$2.'.html';
	} else {
		my $i = rindex($s_str, "\/" );
		my $param = substr($s_str, $i + 1, length($s_str)-1);
		my $front_param = substr($s_str, 0, $i);
		if( $param =~ /(.+)\.(.+)/ ) {								# 拡張子あり
			$path = $s_str;
		} else {
			if( $param =~ /(.+)\#(.+)/ ) {
				$path = $front_param.'/'.$1.'.html#'.$2;			# 拡張子なし#ラベル
			} else {
				$path = $s_str.'.html';								# 拡張子なし
				$second_path = $s_str;
			}
		}
	}
	return($path, $second_path);

}




1;
