#!/usr/bin/perl
#
# MakeDiff.pm
#
# 2010/10/20
#

package MakeDiff;

use warnings;
use strict;
use utf8;
use File::Basename;

# diff config
my $DIFF_CMD       = '/usr/bin/diff';
my $DIFF_HTML_OPT  = "--unchanged-line-format='%L' --new-line-format='<span style=\"border-bottom: 2px solid black;color: red;background-color: yellow;\">%l</span>\n' --old-line-format=''";
my $DIFF_TXT_OPT   = '-y';
my $DIFF_RSS_OPT   = "--unchanged-group-format='' --changed-group-format='\n%>\n'";
my $DIFF_HTML_FILE = 'diff.html';
my $DIFF_TXT_FILE  = 'diff.txt';
my $DIFF_RSS_FILE  = 'rss.frg';

# tidy config
my $TIDY_CMD = '/usr/local/bin/tidy';
my $TIDY_OPT = '-indent -wrap 0 -raw';
my $TIDY_PRE = 'tidy_';


sub make_diff_html() {
	my ($old_file, $new_file) = @_;

	my $rc;
	my $message;
	my $cmd;

	my ($old_name, $old_path) = fileparse($old_file);
	my ($new_name, $new_path) = fileparse($new_file);

	my $old_tidy = "$old_path$TIDY_PRE$old_name";

	unless (-e $old_tidy) {
		($rc, $message) = &exec_tidy($old_file, $old_tidy);
		return (-1, $message) if ($rc == 2);
	}
	
	my $new_tidy = "$new_path$TIDY_PRE$new_name";
	($rc, $message) = &exec_tidy($new_file, $new_tidy);
	return (-1, $message) if ($rc == 2);
	

	my $diff_html = "$new_path$DIFF_HTML_FILE";

	$cmd = "$DIFF_CMD $DIFF_HTML_OPT  $old_tidy $new_tidy > $diff_html";
	$rc  = system("$cmd");

	$rc = $rc >> 8;
	$message = "Error[301] exec diff error : $cmd";
	return (-1, $message) if ($rc == 2);
	
	($rc, $message) = &exec_tidy($diff_html);
	return (-1, $message) if ($rc == 2);

	return 1;
	
}

sub make_diff_txt() {
	my ($old_file, $new_file) = @_;

	my ($old, $new, $new_path) = &get_diff_target($old_file, $new_file);

	my $diff_txt = "$new_path$DIFF_TXT_FILE";

	my $cmd = "$DIFF_CMD $old $new > $diff_txt";
	my $rc  = system("$cmd");

	$rc = $rc >> 8;
	my $message = "Error[301] exec diff error : $cmd";
	return (-1, $message) if ($rc == 2);

	return 1;
}

sub make_diff_rss() {
	my ($old_file, $new_file) = @_;

	my ($old, $new, $new_path) = &get_diff_target($old_file, $new_file);

	my $diff_txt = "$new_path$DIFF_RSS_FILE";

	my $cmd = "$DIFF_CMD $DIFF_RSS_OPT $old $new > $diff_txt";
	my $rc  = system("$cmd");

	$rc = $rc >> 8;
	my $message = "Error[301] exec diff error : $cmd";
	return (-1, $message) if ($rc == 2);

	return 1;
}

sub exec_tidy() {
	my ($orign, $output) = @_;
	my $rc;
	my $cmd;

	if ($output) {
		$cmd = "$TIDY_CMD $TIDY_OPT -o $output $orign 2>/dev/null"; 
		$rc  = system("$cmd");
	} else {
		$cmd = "$TIDY_CMD $TIDY_OPT -m $orign 2>/dev/null";
		$rc  = system("$cmd");
	}
	$rc = $rc >> 8;
	my $message = '';
	
	if ($rc == 2) {
		$message = "Error[300] exec tidy error : $cmd";
	}

	return ($rc,$message);
}

sub get_diff_target() {
	my ($old_file, $new_file) = @_;
	my $old;
	my $new;

	my ($old_name, $old_path) = fileparse($old_file);
	my ($new_name, $new_path) = fileparse($new_file);

	my $old_tidy = "$old_path$TIDY_PRE$old_name";
	my $new_tidy = "$new_path$TIDY_PRE$new_name";

	if (-e $old_tidy && -e $new_tidy) {
		$old = $old_tidy;
		$new = $new_tidy;
	} else {
		$old = $old_file;
		$new = $new_file;
	}
	
	return ($old, $new, $new_path);
}
1;
