#!/usr/bin/perl

use strict;
use warnings;
use lib qw( ./pm );
use CGI;
use common;
use utf8;

&main();
exit(1);

sub main {
	my $query = new CGI;
	
	&common::check_referer($query);
	
	my $history_id      = $query->param('history_id');
	my @select_category = $query->param('category');
	
	# チェックが1つもされていない場合
	@select_category = () if ($select_category[0] eq "");
	
	&update_history($history_id, @select_category);
	
	print $query->header(-type=>'text/html', -charset=>'utf-8');
}

sub update_history {
	my ($history_id, @select_category) = @_;
	
	my $dbh = &common::db_open();
	
	# データベース整合性の確認
	my $id_search_sql  = "select id from tbl_update_histry where id = ?";
	my @id_search_data = ($history_id);
	
	my ($checked_id)
		= &common::count_sql_result($dbh, $id_search_sql, 'exec_category_update.cgi', @id_search_data);
	
	&error_finish($dbh) if ($checked_id == 0);
	
	my $category_delete_sql  = "delete from tbl_category where history_id = ?";
	my @category_delete_data = ($history_id);
	
	my $category_delete_sth  = &common::execute_sql($dbh, $category_delete_sql, 'exec_category_update.cgi',
					 								0, 0, 0, @category_delete_data);
	
	$category_delete_sth->finish();
	
	if (@select_category > 0) {
		foreach my $category (@select_category) {
			# データベース整合性の確認
			my $category_search_sql  = "select code from tbl_category_code where code = ?";
			my @category_search_data = ($category);
			
			my ($checked_category)
				= &common::count_sql_result($dbh, $category_search_sql, 'exec_category_update.cgi', 
											@category_search_data);
			
			&error_finish($dbh) if ($checked_category == 0);
			
			my $category_insert_sql
					= "insert into tbl_category (history_id, category_code) value (?, ?)";
			my @category_insert_data = ($history_id, $category);
			
			my $category_insert_sth  = &common::execute_sql
								($dbh, $category_insert_sql, 'exec_category_update.cgi',
					 			 0, 0, 0, @category_insert_data);
			
			$category_insert_sth->finish();
		}
	} else {
		my $category_insert_sql
					= "insert into tbl_category (history_id, category_code) value (?, '00')";
		my @category_insert_data = ($history_id);
		
		my $category_insert_sth  = &common::execute_sql
							($dbh, $category_insert_sql, 'exec_category_update.cgi',
				 			 0, 0, 0, @category_insert_data);
		
		$category_insert_sth->finish();
	}
	
	$dbh->commit     || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
}

sub error_finish {
	my ($dbh) = @_;

	$dbh->rollback	 || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	my $query = new CGI;
	
	print $query->header(-type=>'text/html', -status=>'400 Bad Request', -charset=>'utf-8');
	
	exit(255);
}
