#!/usr/bin/perl
########################################################################################
# ファイル名：exec_group_delete.cgi
# 機能：選択したグループ番号の行を消去
#
# 改訂履歴：
# 1.0.0 ：新規作成      2009/11/19
# 1.0.1 ：修正　DB更新失敗時のロールバック処理の追加　2010/6/4
########################################################################################
use strict;
use warnings;
use lib qw( ./pm );
use system_config;
use CGI;
use common;
use HTML::Template;
use utf8;

&main();
exit(1);

########################################################################################
# 関数名:main
# 機能  :main処理
# 引数  :なし
# 戻り値:1
########################################################################################
sub main {
	# CGIオブジェクト
	my $query = new CGI;
	
	&common::check_referer($query);
	
	# 引数の取得
	my $id = $query->param('group_id');
	my $option_check = $query->param('option_check');
	
	# DB接続
	my $dbh = &common::db_open();
	&exec_delete($dbh, $id, $option_check);
	
	# ----------------------------------
	# テンプレートのパラメータ設定
	#
	my $template = HTML::Template->new(filename => './template/group_update.tmpl');
	$template->param( COMMENT => $::msg_delete_finish );

	# 返値設定
	print $query->header(-type=>'text/html', -charset=>'utf-8');
	print $template->output;
	
	return(1);
}

########################################################################################
# 関数名:exec_delete
# 機能  :削除対象グループ番号を0に更新または削除し(tbl_itemテーブル)、
#        tbl_item_groupテーブルの対象行を削除する
# 引数  :$dbh, $id, $option_check
# 戻り値:1
########################################################################################
sub exec_delete {
	my($dbh, $group_id, $option_check) = @_;
	my($sth);
	
	if ($option_check eq '1') {
		my $sql = qq { DELETE FROM tbl_item WHERE item_group_id = ? };
		my @data = ($group_id);
		
		$sth = &common::execute_sql($dbh, $sql, 'exec_group_delete.cgi',
					 './template/group_update.tmpl', 'COMMENT', $::msg_db_error2, @data);
	} elsif ($option_check eq '0') {
		my $sql = qq { UPDATE tbl_item SET item_group_id = 0 WHERE item_group_id = ? };
		my @data = ($group_id);

		$sth = &common::execute_sql($dbh, $sql, 'exec_group_delete.cgi',
					 './template/group_update.tmpl', 'COMMENT', $::msg_db_error2, @data);
	} else {
		$dbh->rollback || warn $dbh->errstr;
		$dbh->disconnect || warn $dbh->errstr;
		
		my $query = new CGI;
		print $query->header(-type=>'text/html', -status=>'400 Bad Request', -charset=>'utf-8');
		exit (255);
	}


	# SQL作成
	my $sql = qq{ DELETE FROM tbl_item_group WHERE item_group_id = ? };
	my @data = ($group_id);
	
	$sth = &common::execute_sql($dbh, $sql, 'exec_group_delete.cgi',
					 './template/group_update.tmpl', 'COMMENT', $::msg_db_error2, @data);

	# 検索終了(SQL開放)
	#########################################
	$sth->finish;
	
	# コミット ＆切断
	#########################################
	$dbh->commit || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	return(1);
}
