#!/usr/bin/perl
########################################################################################
# ファイル名：item_update.cgi
# 機能：引数の内容と登録情報が異なる場合にアイテムを更新
#
# 改訂履歴：
# 1.0.0 ：新規作成      2009/11/12
# 1.0.1 ：バグ修正　URLが変更された場合「tbl_sub_item」テーブルのDELETEを実行する。
#         キー：item_id　　　　　　　　　　　　　　　　　　2010/6/4
# 
########################################################################################
use strict;
use warnings;
use lib qw( ./pm );
use system_config;
use CGI;
use common;
use HTML::Template;
use utf8;

&main();
exit(1);

########################################################################################
# 関数名:main
# 機能  :main処理
# 引数  :なし
# 戻り値:1
########################################################################################
sub main {
	# CGIオブジェクト
	my $query = new CGI;
	
	&common::check_referer($query);
	
	my $result_comment;
	
	# 引数の取得
	my $item = $query->param('item');
	my $item_id = $query->param('item_id');
	my $url = $query->param('url');
	my $link = $query->param('follow_link');
	my $name = $query->param('name');
	my $password = $query->param('password');
	my $item_group = $query->param('item_group');
	my $medals_id = $query->param('medals_id');
	my $comment = $query->param('comment');
	
	# データの確認
	my $cnt = &get_item_info($item_id, $item, $url, $link, $name, $password, $item_group, $medals_id, $comment);
	
	if($cnt == 0) {
		# DB接続
		my $dbh = &common::db_open();
		&tbl_item_update($dbh, $item, $item_id, $url, $link, $name, $password, $item_group, $medals_id, $comment);
		$result_comment = $::msg_update_finish;
	} else {
		$result_comment = $::msg_same_data;
	}
	# ----------------------------------
	# テンプレートのパラメータ設定
	#
	my $template = HTML::Template->new(filename => './template/update.tmpl');
	$template->param( COMMENT => $result_comment );

	# 返値設定
	print $query->header(-type=>'text/html', -charset=>'utf-8');
	print $template->output;
	
	return(1);
}

########################################################################################
# 関数名:get_item_info
# 機能  :更新データが元のデータと同じか否かをチェック
# 引数  :$dbh, $item, $item_id, $url, $link, $name, $password, $item_group, $comment
# 戻り値:ヒット件数　(0→更新対象データ　1→同じデータ(更新しない))
########################################################################################
sub get_item_info {
	my($item_id);
	my($item);
	my($url);
	my($link);
	my($name);
	my($password);
	my($group);
	my($medals_id);
	my($comment);
	my(@param);
	
	my($cnt);
	my($sth);
	
	# 引数取得
	$item_id = shift(@_);
	$item = shift(@_);
	$url = shift(@_);
	$link = shift(@_);
	$name = shift(@_);
	$password = shift(@_);
	$group = shift(@_);
	$medals_id = shift(@_);	
	$comment = shift(@_);
	
	# 入力情報での実行に更新内容があるかを確認するSQL文の作成
	# ある場合…cnt=1, ない場合…cnt=0
	my $dbh = &common::db_open();
	my $sql = qq{ SELECT item_id FROM tbl_item WHERE item_id = ? AND item_name = ? AND item_URL = ?
				AND follow_link = ? AND user_id = ? AND password = ? 
				AND medals_id = ? AND comment = ? AND item_group_id = ? };
	my @data = ($item_id, $item, $url, $link, $name, $password, $medals_id, $comment, $group);

	($cnt, $sth) = &common::count_sql_result($dbh, $sql, 'item_update.cgi', @data);
	
	# 検索終了(SQL開放)
	#########################################
	$sth->finish;
	
	# コミット＆切断
	#########################################
	$dbh->commit || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	return($cnt);

}

########################################################################################
# 関数名:tbl_item_update
# 機能  :tbl_itemテーブルの更新
# 引数  :$dbh, $item, $item_id, $url, $link, $name, $password, $item_group, $comment
# 戻り値:1
########################################################################################
sub tbl_item_update {
	my($sth);

	my ($dbh, $item, $item_id, $url, $link, $name, $password, $item_group, $medals_id, $comment) = @_;

	my $same_url_sql = qq { select * from tbl_item where item_id = ? and item_URL = ? };
	my $cnt;
	($cnt,$sth) = &common::count_sql_result($dbh, $same_url_sql, 'item_update.cgi', $item_id, $url);

	my $sql = qq{UPDATE tbl_item SET item_name = ?, item_URL = ?, follow_link = ?,
			 user_id = ?, password = ?, medals_id = ?, comment = ?, item_group_id = ? WHERE item_id = ? };
	my @data = ($item, $url, $link, $name, $password, $medals_id, $comment, $item_group, $item_id);
	
	$sth = &common::execute_sql($dbh, $sql, 'item_update.cgi', 
						'./template/update.tmpl', 'COMMENT', $::msg_db_error1, @data);
	#tbl_sub_itemのDELETE
	if ($cnt == 0) {
		my $d_sql = qq { DELETE FROM  tbl_sub_item WHERE item_id = ?};
		my @d_data = ($item_id);
		$sth = &common::execute_sql($dbh, $d_sql, 'item_update.cgi',
						'./template/update.tmpl', 'COMMENT', $::msg_db_error1, @d_data);
	}
	
	# 検索終了(SQL開放)
	#########################################
	$sth->finish;
	
	# コミット or ロールバック ＆切断
	#########################################
	$dbh->commit || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	
	return(1);
}
