#!/usr/bin/perl

# ファイル名：search_result.cgi
# 機能：検索結果テーブルを出力。
#
# 改訂履歴：
# 1.0.0 ：新規作成      2009/11/11
# 

use strict;
use warnings;
use lib qw( ./pm );
use system_config;
use CGI;
use common;
use HTML::Template;
use utf8;

&main();
exit(1);

# 関数名:main
# 機能  :main処理
# 引数  :なし
# 戻り値:1
sub main {
	my $query = new CGI;
	
	# パラメータ取得 検索キー
	my $keyword = &common::check_string( $query->param("keyword") );
	my $mode = &common::check_number ( $query->param("mode") );
	my $item_group_id = &common::check_number ( $query->param("item_group") );
	my $start = &common::check_number ( $query->param("start") );
	my $end = &common::check_number ( $query->param("end") );
	
	# 異常入力に対するエラー処理
	if ($mode !~ /[1-4]/ ||
		($mode == 4 && $keyword !~ /\d+/) ||
		($mode != 4 && length($keyword) < 2)) { 
		print $query->header(-type => 'text/html', -status => '400 Bad request', -charset => 'utf-8');
		exit(1);
	}
	
	my $dbh = &common::db_open();
	
	my ($sql, @data) = &make_sql($keyword, $mode, $item_group_id, $start, $end);

	my ($result, $count) = &get_search_result($sql, $dbh, @data);

	# テンプレートのパラメータ設定
	my $template = HTML::Template->new(filename => './template/result.tmpl',
					loop_context_vars => 1);
	$template->param(RESULT => $result);
	$template->param(COUNT => $count);
	$template->param(LOGIN_USR => $::login_usr);

	# 画面出力, HTML生成
	print $query->header(-type => 'text/html', -charset => 'utf-8');
	print $template->output;
}

# 関数名: get_search_result
# 機能  : アイテム情報の取得
# 引数  : sql文, データベースハンドラー, sqlに利用するデータ格納リスト
# 戻り値: 検索結果
sub get_search_result {
	my ($sql, $dbh, @data) = @_;
	
	my @loop_data = ();
	my $count = 0;

	my $sth = &common::execute_sql($dbh, $sql, 'search_result.cgi', -1, -1, -1, @data);

	while(my @row = $sth->fetchrow) {
		my %row_data;
		$row_data{item_id}		= $row[0];
		$row_data{item}			= &common::HTMLEscape($row[1]);
		$row_data{url}			= &common::HTMLEscape($row[2]);
		$row_data{follow_link} 	= $row[3];
		$row_data{user} 		= $row[4];
		$row_data{password} 	= $row[5];
		$row_data{comment} 		= &common::HTMLEscape($row[6]);
		$row_data{group_id} 	= $row[7];
		$row_data{group_name} 	= $row[8];
		$row_data{medals_id}	= $row[9];
		
		push(@loop_data, \%row_data);
	}

	$dbh->commit     || warn $dbh->errstr;
	$dbh->disconnect || warn $dbh->errstr;
	$count = @loop_data;
	return (\@loop_data, $count);
}

# 関数名: make_sql
# 機能  : sql文の作成
# 引数  : $keyword, $mode, $item_group_id, $start, $end
# 戻り値: sql文, プリペア数値格納リスト
sub make_sql {
	my($keyword, $mode, $item_group_id, $start, $end) = @_;
	
	my $sql;
	my @data = ();
	
	# SQL文の作成
	# ItemID検索時
	if ($mode == 4){
		$sql .= qq { SELECT distinct i.item_id, i.item_name, i.item_URL, i.follow_link, i.user_id, i.password, i.comment, i.item_group_id, g.item_group_name FROM tbl_item i left join tbl_item_group g on i.item_group_id = g.item_group_id WHERE i.item_id = ? };

		push(@data, $keyword);

		return ($sql, @data);
	}
	# ItemID検索時以外
	$sql .= qq { SELECT distinct i.item_id, i.item_name, i.item_URL, i.follow_link, i.user_id, i.password, i.comment, i.item_group_id, g.item_group_name, i.medals_id FROM ((tbl_item i left join tbl_item_group g on i.item_group_id = g.item_group_id) left join tbl_update_histry u on i.item_id = u.item_id) };

	my $serch_word = &common::keyword_quote($keyword);
	push(@data, $serch_word);

	# modeに関する追加文
	if ($mode == 1) {
		$sql .= qq { WHERE i.item_name  LIKE ? };
	} elsif ($mode == 2) {
		$sql .= qq { WHERE i.item_URL  LIKE ? };
	} else {
		$sql .= qq { WHERE i.comment LIKE ? };
	}

	# item_group_idに関する追加文
	if($item_group_id ne "all") {
		$sql .= qq { AND i.item_group_id = ? };
		push(@data, $item_group_id);
	}

	# startに関する追加文
	if($start =~ /\d{8}/) {
		$sql .= qq { AND u.date_update >= ? };
		push(@data, $start);
	}

	# endに関する追加文
	if($end =~ /\d{8}/) {
		$sql .= qq { AND u.date_update <= ? };
		push(@data, $end);
	}

	$sql .= qq{ ORDER BY item_name ASC };

	return ($sql, @data);
}
